<?php

class RunScheduleController extends Controller
{
  protected function beforeAction($event)
  {
    access_denied_check("run_schedule_management");
    return parent::beforeAction($event);
  }
  
  /**
   * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
   * using two-column layout. See 'protected/views/layouts/column2.php'.
   */
  public $layout='//layouts/column2';

  /**
   * Displays a particular model.
   * @param integer $id the ID of the model to be displayed
   */
  public function actionView($id)
  {
    $this->render('view',array(
        'model'=>$this->loadModel($id),
    ));
  }

  public function actionSwitchActive($id=null, $returnUrl=null)
  {
    access_denied_check("run_schedule_update");
    $model = $this->loadModel($id);
    $model->active = $model->active == 1 ? 0 : 1;
    $model->save();
    $this->redirect(empty($returnUrl) ? url("runSchedule/admin") : $returnUrl);
  }
  
  public function actionSwitchConfirmationRequired($id=null, $returnUrl=null)
  {
    access_denied_check("run_schedule_update");
    $model = $this->loadModel($id);
    $model->confirmation_required = $model->confirmation_required == 1 ? 0 : 1;
    $model->save();
    $this->redirect(empty($returnUrl) ? url("runSchedule/admin") : $returnUrl);
  }

  /**
   * Creates a new model.
   * If creation is successful, the browser will be redirected to the 'view' page.
   */
  public function actionCreate()
  {
    access_denied_check("run_schedule_create");
    $model=new RunSchedule;

    // Uncomment the following line if AJAX validation is needed
    // $this->performAjaxValidation($model);

    if(isset($_POST['RunSchedule']))
    {
      $model->attributes=$_POST['RunSchedule'];
      if($model->save())
        $this->redirect(array('view','id'=>$model->ID));
    }

    $this->render('create',array(
        'model'=>$model,
    ));
  }

  /**
   * Updates a particular model.
   * If update is successful, the browser will be redirected to the 'view' page.
   * @param integer $id the ID of the model to be updated
   */
  public function actionUpdate($id)
  {
    access_denied_check("run_schedule_update");
    $model=$this->loadModel($id);

    // Uncomment the following line if AJAX validation is needed
    // $this->performAjaxValidation($model);

    if(isset($_POST['RunSchedule']))
    {
      $model->attributes=$_POST['RunSchedule'];
      if($model->save())
        $this->redirect(array('view','id'=>$model->ID));
    }

    $this->render('update',array(
        'model'=>$model,
    ));
  }

  /**
   * Deletes a particular model.
   * If deletion is successful, the browser will be redirected to the 'admin' page.
   * @param integer $id the ID of the model to be deleted
   */
  public function actionDelete($id)
  {
    access_denied_check("run_schedule_delete");
    if(Yii::app()->request->isPostRequest)
    {
      // we only allow deletion via POST request
      $this->loadModel($id)->delete();

      // if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
      if(!isset($_GET['ajax']))
        $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
    }
    else
      throw new CHttpException(400,'Invalid request. Please do not repeat this request again.');
  }

  /**
   * Lists all models.
   */
  public function actionIndex()
  {
    $this->redirect(url('RunSchedule/admin'));
  }

  /**
   * Manages all models.
   */
  public function actionAdmin()
  {
    $model=new RunSchedule('search');
    $model->unsetAttributes();  // clear any default values
    if(isset($_GET['RunSchedule']))
      $model->attributes=$_GET['RunSchedule'];

    $this->render('admin',array(
        'model'=>$model,
    ));
  }

  /**
   * Returns the data model based on the primary key given in the GET variable.
   * If the data model is not found, an HTTP exception will be raised.
   * @param integer the ID of the model to be loaded
   * @return RunSchedule object
   */
  public function loadModel($id)
  {
    $model=RunSchedule::model()->findByPk($id);
    if($model===null)
      throw new CHttpException(404,'The requested page does not exist.');
    return $model;
  }

  /**
   * Performs the AJAX validation.
   * @param CModel the model to be validated
   */
  protected function performAjaxValidation($model)
  {
    if(isset($_POST['ajax']) && $_POST['ajax']==='run-schedule-form')
    {
      echo CActiveForm::validate($model);
      Yii::app()->end();
    }
  }
}
