<?php

/**
 * This is the model class for table "device_type".
 *
 * The followings are the available columns in table 'device_type':
 * @property integer $id
 * @property string $name
 * @property string $type
 */
class DeviceType extends ActiveRecord
{
	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return DeviceType the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'device_type';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('name, type', 'required'),
			array('name, type', 'length', 'max'=>30),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, name, type', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function myAttributeLabels()
	{
		return array(
			'id' => 'ID',
			'name' => 'Name',
			'type' => 'Type',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('name',$this->name,true);
		$criteria->compare('type',$this->type,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}
	
	public static function getTypeList()
	{	/** Mod. Cache Queries 18.02.2019 Ng. */ 
		if (Yii::app()->params['cached_queries']) {
		$devName = DeviceType::model()->cache(1000)->findAll(array('select'=>'name', 'distinct'=> true));
		}else{
			$devName = DeviceType::model()->findAll(array('select'=>'name', 'distinct'=> true));
		}
		if(is_null($devName))
			return null;
		
		if(!is_array($devName))
			$devName = array($devName);
		
		$arr = array();
		foreach($devName as $dN)
			$arr[$dN->name] = $dN->name;
		
		return $arr;

	}
}